;------------------------------------------------------------------------
;-  Written by: Neviksti
;-     This was coded as an example for Darrok (from the zsnes message
;-     boards) who was interested in learning more about the SNES 
;-     hardware and language.
;-
;-     If you use my code, please share your creations with me
;-     as I am always curious :)
;------------------------------------------------------------------------

;=== Include MemoryMap, VectorTable, HeaderInfo ===
.INCLUDE "walker.inc"

;=== Include Library Routines & Macros ===
.INCLUDE "LoadGraphics.asm"
.INCLUDE "InitSNES.asm"
.INCLUDE "2input.asm"
.INCLUDE "Sprites.asm"
.INCLUDE "Collision.asm"

;==============================================================================
; main
;==============================================================================

.DEFINE MapX		$18
.DEFINE MapY		$1A

.BANK 0 SLOT 0
.ORG 0
.SECTION "WalkerCode"

Main:
	InitializeSNES

	rep #$10		;A/mem = 8bit, X/Y=16bit
	sep #$20

      
	;Load palette to make our pictures look correct
	LoadPalette	BG_Palette, 0, 128
	LoadPalette	RedPalette, 128, 9
	LoadPalette	GreenPalette, 144, 9
	LoadPalette	BluePalette, 160, 9

	;Load Tile and Character data to VRAM
	LoadBlockToVRAM	SpriteTiles, $0000, $0040, $0000
	LoadBlockToVRAM	SpriteTiles, $0100, $0040, $0040
	LoadBlockToVRAM	BackgroundPics, $4000, $6000, $0000	; 384 tiles * (8bit color)= 0x6000 bytes
	LoadBlockToVRAM	BackgroundMap, $7800, $1000, $0000	; 64x64 tiles = 4096 words = 8192 bytes

	JSR SpriteInit	;setup the sprite buffer
	JSR JoyInit		;setup joypads and enable NMI

	;setup our walking sprite

	;put sprite #0 on screen
	lda #%01010000
	sta SpriteBuf2

	;put him in the center of the screen
	lda #$00 		
	sta SpriteBuf1+0
	lda #(224/2-16)
	sta SpriteBuf1+1

	lda #$00
	sta SpriteBuf1+2

	;set the sprite to the highest priority
	lda #$30			
	sta SpriteBuf1+3

	;put him in the center of the screen
	lda #$78 		
	sta SpriteBuf1+4
	lda #(224/2-16)
	sta SpriteBuf1+5

	lda #$00
	sta SpriteBuf1+6

	;set the sprite to the highest priority
	lda #$34			
	sta SpriteBuf1+7

	;setup the video modes and such, then turn on the screen
	JSR SetupVideo	

InfiniteLoop:

	WAI

	jsr	CollisionCode

	;See what buttons were pressed

	lda	Joy1+1
	and	#$08
	beq	_down

	lda	SpriteBuf1+1
	cmp	#$04
	beq	_down
	sec
	sbc	#$02
	sta	SpriteBuf1+1

_down:
	lda	Joy1+1
	and	#$04
	beq	_left

	lda	SpriteBuf1+1
	cmp	#$BC
	beq	_left
	clc
	adc	#$02
	sta	SpriteBuf1+1

_left:
	lda	Joy1+1
	and	#$02
	beq	_right

	lda	SpriteBuf1+0
	cmp	#$04
	beq	_right
	sec
	sbc	#$02
	sta	SpriteBuf1+0

_right:
	lda	Joy1+1
	and	#$01
	beq	_done

	lda	SpriteBuf1+0
	cmp	#$DC
	beq	_done	
	clc
	adc	#$02
	sta	SpriteBuf1+0

_done
	
	JMP InfiniteLoop	;Do this forever

;==========================================================================================

.DEFINE FrameNum $12		

VBlank:
	rep #$30		;A/Mem=16bits, X/Y=16bits
	phb
	pha
	phx
	phy
	phd

	sep #$20		; mem/A = 8 bit, X/Y = 16 bit

	;*********transfer sprite data

	stz $2102		; set OAM address to 0
	stz $2103

	LDY #$0400
	STY $4300		; CPU -> PPU, auto increment, write 1 reg, $2104 (OAM data write)
	LDY #$0400
	STY $4302		; source offset
	LDY #$0220
	STY $4305		; number of bytes to transfer
	LDA #$7E
	STA $4304		; bank address = $7E  (work RAM)
	LDA #$01
	STA $420B		;start DMA transfer

	;update the map co-ordinates
	lda MapX
	sta $210D
	lda MapX+1
	sta $210D

	lda MapY
	sta $210E
	lda MapY+1
	sta $210E

	;update the joypad data
	JSR GetInput

	lda $4210		;clear NMI Flag

	REP #$30		;A/Mem=16bits, X/Y=16bits
	
	inc FrameNum

	PLD 
	PLY 
	PLX 
	PLA 
	PLB 
      RTI

;End of demo Main code

;============================================================================
; SetupVideo -- Set the video mode for the demo
;----------------------------------------------------------------------------
; In: None
;----------------------------------------------------------------------------
; Out: None
;----------------------------------------------------------------------------
SetupVideo:
	php

	rep #$10		;A/mem = 8bit, X/Y=16bit
	sep #$20
      
	lda #%01100000		;Sprites 32x32 or 64x64, character data at $0000 (word address)
      sta $2101         

	lda #$03		;Set video mode 3, 8x8 tiles (256 color BG1, 16 color BG2)
      sta $2105         

	lda #%01111110		;Set BG1's Tile Map VRAM offset to $7800 (word address)
      sta $2107		;   and the Tile Map size to 32 tiles x 32 tiles

	lda #$04		;Set BG1's Character VRAM offset to $4000 (word address)
      sta $210B

	lda #$11		;Turn on BG1 and BG2 and Sprites
      sta $212C

      lda #$0F		;Turn on screen, full brightness
      sta $2100		

	lda #$FF		;Scroll BG2 down 1 pixel
	sta $2110
	sta $2110         

	plp
	rts

.ENDS

;==========================================================================================

.BANK 1 SLOT 0
.ORG 0
.SECTION "CharacterData"

;Map data
BackgroundMap:
	.INCBIN ".\\GamePictures\\mymap.map"

;Color data
BG_Palette:
	.INCBIN ".\\GamePictures\\mymap.clr"

RedPalette
	.INCBIN ".\\GamePictures\\red.clr"

GreenPalette
	.INCBIN ".\\GamePictures\\green.clr"

BluePalette
	.INCBIN ".\\GamePictures\\blue.clr"

;Tile data
SpriteTiles:
	.INCBIN ".\\GamePictures\\square.pic"

.ENDS

;==========================================================================================

.BANK 4 SLOT 0
.ORG 0
.SECTION "BG_CharacterData"

;character data
BackgroundPics:
	.INCBIN ".\\GamePictures\\mymap.pic"
.ENDS

;==========================================================================================
